"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _path2 = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isTrueRequire(path) {
  return path.node.type === 'CallExpression' && path.node.callee.type === 'Identifier' && path.node.callee.name === 'require' && !path.scope.hasBinding('require');
}

function _default() {
  let lastOpts;
  let lastCwd;
  let lastPatterns;

  function selectPatterns(state) {
    const {
      opts
    } = state;
    const cwd = state.cwd || process.cwd();
    if (opts === lastOpts && cwd === lastCwd) return lastPatterns;
    lastCwd = cwd;
    lastOpts = opts;
    const {
      packages = [],
      patterns = [],
      paths = []
    } = opts;

    if (!Array.isArray(packages)) {
      throw new Error('packages must be an array if given');
    }

    if (!Array.isArray(patterns)) {
      throw new Error('patterns must be an array if given');
    }

    if (!Array.isArray(paths)) {
      throw new Error('paths must be an array if given');
    }

    return lastPatterns = [...packages.map((pkg, index) => {
      if (typeof pkg !== 'string') {
        throw new Error(`packages[${index}] must be a string`);
      }

      return {
        pattern: new RegExp(`^${pkg}(/|$)`),
        type: 'package',
        what: pkg
      };
    }), ...patterns.map((_pattern, index) => {
      let pattern;

      if (Array.isArray(_pattern)) {
        if (typeof _pattern[0] !== 'string') {
          throw new Error(`patterns[${index}][0] must be a string`);
        }

        if (_pattern.length > 1 && typeof _pattern[1] !== 'string') {
          throw new Error(`patterns[${index}][1] must be a string if given`);
        }

        pattern = new RegExp(_pattern[0], _pattern[1] || '');
      } else if (typeof _pattern === 'string') {
        pattern = new RegExp(_pattern);
      } else {
        throw new Error(`patterns[${index}] must be a string or array of strings`);
      }

      return {
        pattern,
        type: 'pattern',
        what: pattern
      };
    }), ...paths.map((_path, index) => {
      if (typeof _path !== 'string') {
        throw new Error(`paths[${index}] must be a string`);
      }

      return {
        pattern: new RegExp(`^${_path2.default.resolve(cwd, _path)}(/|$)`),
        type: 'path',
        what: _path
      };
    })];
  }

  function test(rawSource, state) {
    const filename = state.file && state.file.opts && state.file.opts.filename || state.filename;
    const cwd = state.cwd || process.cwd();
    const source = /^[./]/.test(rawSource) ? _path2.default.resolve(_path2.default.dirname(filename), rawSource) : rawSource;
    const patterns = selectPatterns(state);
    patterns.forEach(({
      pattern,
      type,
      what
    }) => {
      if (pattern.test(source)) {
        let rule;

        switch (type) {
          case 'package':
            rule = `'${what}'`;
            break;

          case 'pattern':
            rule = String(what);
            break;

          case 'path':
            rule = _path2.default.relative(_path2.default.dirname(filename), _path2.default.resolve(cwd, what));
            if (!/^\./.test(rule)) rule = `./${rule}`;
            rule = `'${rule}'`;
            break;
        }

        throw new Error(`importing from ${rule} is forbidden (imported: '${rawSource}')`);
      }
    });
  }

  return {
    visitor: {
      ImportDeclaration(path, state) {
        test(path.node.source.value, state);
      },

      CallExpression(path, state) {
        if (path.node.callee.type === 'Import' || isTrueRequire(path)) {
          test(path.node.arguments[0].value, state);
        }
      }

    }
  };
}